function [IAE_2DZ_Tdm,param_2DZ_Tdm,error_2DZ_Tdm,IAE_2D,param_2DZ,error_2D] = fdomain_simulate_criteria(num,den,Tdelay)
%[IAE_2DZ_Tdm,param_2DZ_Tdm,error_2DZ_Tdm,IAE_2D,param_2DZ,error_2D] = calc_fdomain_sim_criteria(num,den,Tdelay)
%
% Function calculates second-order process model parameters from
% characteristic areas with proposed method (five-parameters) and method
% proposed by Vrecko et al. (four-parameters -> no zero).
%
% Kpr*(1 + b*s)*e-Tdm*s
% ---------------------
%   a2*s^2 + a1*s + 1
%
% INPUTS:
% num:          Process numerator.
% den:          Process denominator.
% Tdelay:       Process delay (dead time).
%
% OUTPUTS:
% IAE_2DZ_Td        IAE of the proposed (five-parameters) method.
% param_2DZ_Tdm     Parameters [Kpr a1 a2 b1 Tdm] of the proposed method.
% error_2DZ_Tdm     Errors of the proposed method.
% IAE_2D            IAE of the Vrecko et al. (four-parameters) method.
% param_2DZ         Parameters [Kpr a1 a2 Tdm] of the Vrecko et al. method.
% error_2D          Errors of the Vrecko et al. method.

t = 0:0.01:10;
size1 = max(size(t));
Ts = (t(size1)-t(1))/(size1-1);

P_tf = tf(num,den,'InputDelay',Tdelay);
[p_res_original] = step(P_tf,t);

[A0,A1,A2,A3,A4,A5] = numden2areas (num,den,Tdelay);
if 1<length(num) & 0>num(1)
    process_type = "non-minimum phase";
else
    process_type = "minimum phase";
end

% Proposed method
[Kpr_new,a1_new,a2_new,b1_new,Tdelay_new,error_2DZ_Tdm] = area2Model_2DZ_Tdm(A0,A1,A2,A3,A4,process_type);
param_2DZ_Tdm = [Kpr_new a1_new a2_new b1_new Tdelay_new];
if ( false == error_2DZ_Tdm(3) & false == error_2DZ_Tdm(4) )
    num_new=[Kpr_new*b1_new Kpr_new];
    den_new=[a2_new a1_new 1];
    P_tf = tf(num_new,den_new,'InputDelay',Tdelay_new);
    [p_res_calc] = step(P_tf,t);
else
    p_res_calc = NaN(length(t),1);
end

% Method proposed by Vrecko et al.
% D. Vrecko, D. Vrancic, D. Juricic, and S. Strmcnik, A new modified Smith
% predictor: the concept, design and tuning, ISA Transactions, vol. 40,
% no. 2, pp. 111121, Apr. 2001, doi: 10.1016/S0019-0578(00)00039-2.
[Kpr_old,a1_old,a2_old,Tdelay_old,error_2D] = area2Model2D(A0,A1,A2,A3);
param_2DZ = [Kpr_old a1_old a2_old Tdelay_old];
if ( false==error_2D(2) )
    num_old=[Kpr_old];
    den_old=[a2_old a1_old 1];
    P_tf = tf(num_old,den_old,'InputDelay',Tdelay_old);
    [p_res_2D] = step(P_tf,t);
else
    p_res_2D = NaN(length(t),1);
end

% calculate criteria
err = p_res_original-p_res_calc;
IAE_2DZ_Tdm = ( sum(abs(err))*Ts )/abs( max(p_res_original)-min(p_res_original) );

err = p_res_original-p_res_2D;
IAE_2D = ( sum(abs(err))*Ts )/abs( max(p_res_original)-min(p_res_original) );

end