% This .m file tests proposed model identification method in the
% frequency-domain (identification of the model from a general-order
% transfer function with a time-delay). Used in Section 4.1 of the paper.

clear all;
close all;

% additionals folders
addpath('func_MAIN');
addpath('SubScripts');
addpath('func_plot');
addpath('func_ExportFigure');

%image save
image_dir = '_Image_export_emf_';
image_name = 'Test_Fdomain';
if ~exist(fullfile(pwd, image_dir), 'dir')
    mkdir(fullfile(pwd, image_dir));
end

%% Process pertubation test (frequency-domain)

% Create vectors for Process pertubation
n_v = 1:1:8;
Tdelay_v = logspace(-3,0,30); Tdelay_v = Tdelay_v(1:length(Tdelay_v)-1);
zero_v = linspace(-0.5,0.5,length(Tdelay_v));

disp('Process pertubation test started. Process zero:');
for z=1:1:length(zero_v)
    b = zero_v(z);
    disp(num2str(b));
    num = [b 1];
    
    for y=1:1:length(n_v)
        n = n_v(y);

        for x=1:1:length(Tdelay_v)
            Tdelay = Tdelay_v(x);

            T = (1-Tdelay)/n; % T*n + Tdelay = 1
            den_1 = [T 1];
            den = 1;
            for a=1:n
                den = conv(den,den_1);
            end

            % Calculate model parameters and simulate
            [IAE_2DZ_Tdm(y,x,z),param_2DZ_Tdm,error_2DZ_Tdm,IAE_2D(y,x,z),param_2DZ,error_2D] =...
                fdomain_simulate_criteria(num,den,Tdelay);
            
            % Prepare data for calculated parameters
            b_calc(y,x,z) = param_2DZ_Tdm(4);
            Tdm_calc(y,x,z) = param_2DZ_Tdm(5);

            % Check errors
            err_tmp = [];
            if ( (error_2DZ_Tdm(1) & error_2DZ_Tdm(2)) | (error_2DZ_Tdm(1) & error_2DZ_Tdm(5)) ||...
                    (error_2DZ_Tdm(1) & error_2DZ_Tdm(6)) | (error_2DZ_Tdm(2) & error_2DZ_Tdm(5)) ||...
                    (error_2DZ_Tdm(2) & error_2DZ_Tdm(6)) | (error_2DZ_Tdm(5) & error_2DZ_Tdm(6)) )
                warning('PROPOSED: Multiple algorithms conditions detected!');
            end
            if ( error_2DZ_Tdm(3) )
                warning('PROPOSED: Process model not calculated!');
            end
            if ( error_2DZ_Tdm(4) )
                warning('PROPOSED: More than one solution found!');
            end
            if ( error_2D(2) )
                warning('VRECKO et al.: Process model not calculated!');
            end
            
            % Prepare data for algorithm conditions
            tmp = 0;            
            if ( error_2DZ_Tdm(1) ) % Tdm0 set to 0.
                tmp = 1;
            elseif ( error_2DZ_Tdm(2) ) % Tdm cannot be calculated from sixt-order polynomial
                tmp = 2;
            elseif ( error_2DZ_Tdm(6) ) % Process type required
                tmp = 3;
            elseif ( error_2DZ_Tdm(5) ) % Calculation loop run twice.
                tmp = 4;                
            end
            cond_m(y,x,z) = tmp;
        end
    end
end
[X, Y, Z] = meshgrid(Tdelay_v, n_v, zero_v);  % form the 3D grid

% Plot: Algorithm conditions vs. process parameters
for im=0:1:0
    range = unique(cond_m);
        
    [P_subplot,P_fig,P_colorbar] = subplot_position(460,380,1,1,2,5,5,25,12,50,50);
    figure('Name',char(strcat(image_name,{' '},'conditions')),'NumberTitle','off','Position',P_fig);
    subplot('position',P_subplot{1});
    scatter3(X(:), Y(:), Z(:), 12, cond_m(:), 'filled','MarkerEdgeAlpha', 1, 'MarkerFaceAlpha', 1);
    view(-82.221699999999998,32.591788594164463);
    grid on;
    set(gca,'XMinorTick','on','ZMinorTick','on');
    set(gca,'xscale','log');
    h = xlabel('{\it T}_d_e_l_a_y [s]'); 
    h.Position = [0.035984152162232,-0.077100576068925,-0.503587218538343];
    h = ylabel('{\it n} [/]');
    h.Position = [0.000738124883731,4.318736427958505,-0.573604032950502];
    ylim([min(n_v) max(n_v)]);
    h = zlabel('{\it b} [/]');
    h.Position = [0.000922042789652,8.602126806647092,3.57627869e-7];
    zticks([-0.5 -0.25 0 0.25 0.5]);
    
    colormap(lines(length(range)));
    P_colorbar(1) = P_colorbar(1)-0.02;
    P_colorbar(2) = P_colorbar(2)+0.125;
    c = colorbar('vert','Position',P_colorbar);
    set(c, 'ticks', range);
    middle = (range(end)-range(1))/2+range(1);
    move = (middle-range(1))/(length(range)/2);
    start = middle-((length(range)-1)/2)*move;
    for i = 1:length(range)
        range_p(i) = start+(i-1)*move;
    end
    levellist_s={'/','D = NO','G = NO','J = YES','L = NO'};
    set(c,'YTick',single(round(range_p,2)),'YTicklabel',levellist_s,'TickLength',0);

    h = title('Algorithm conditions vs. process parameters', 'FontWeight','Normal');
    h.Position(1) = h.Position(1)-0.1;

    tightfigadv();
    
    img_save_name = strcat(char(image_name),'_','algorithm');
    export_figure_as_vecgra([], fullfile(pwd, image_dir, char(img_save_name)), '-dmeta');
    export_figure_as_vecgra([], fullfile(pwd, image_dir, char(img_save_name)), '-dpdf');
end
% Plot: Calculated b1m vs. process parameters
for im=0:1:0
    range = linspace(-0.5,0.5,9);
    
    [P_subplot,P_fig,P_colorbar] = subplot_position(460,380,1,1,2,5,5,30,12,50,50);
    figure('Name',char(strcat(image_name,{' '},'calculated {\it b}_1_m')),'NumberTitle','off','Position',P_fig);
    subplot('position',P_subplot{1});
    scatter3(X(:), Y(:), Z(:), 12, b_calc(:), 'filled','MarkerEdgeAlpha', 1, 'MarkerFaceAlpha', 1);
    view(-82.221699999999998,32.591788594164463); 
    grid on;
    set(gca,'XMinorTick','on','ZMinorTick','on');
    set(gca,'xscale','log');
    h = xlabel('{\it T}_d_e_l_a_y [s]'); 
    h.Position = [0.035984152162232,-0.077100576068925,-0.503587218538343];
    h = ylabel('{\it n} [/]');
    h.Position = [0.000738124883731,4.318736427958505,-0.573604032950502];
    ylim([min(n_v) max(n_v)]);
    h = zlabel('{\it b} [/]');
    h.Position = [0.000922042789652,8.602126806647092,3.57627869e-7];
    zticks([-0.5 -0.25 0 0.25 0.5]);
    
    colormap(jet);
    P_colorbar(1) = P_colorbar(1)-0.01;
    P_colorbar(2) = P_colorbar(2)+0.125;
    c = colorbar('vert','Position',P_colorbar);
    set(c, 'ticks', range);
    title(c,'{\it b}_1_m [/]','FontWeight','Normal');
    
    h = title('Calculated {\it b}_1_m vs. process parameters', 'FontWeight','Normal');
    h.Position(1) = h.Position(1)-0.2;

    tightfigadv();
    
    img_save_name = strcat(char(image_name),'_','b1m');
    export_figure_as_vecgra([], fullfile(pwd, image_dir, char(img_save_name)), '-dmeta');
    export_figure_as_vecgra([], fullfile(pwd, image_dir, char(img_save_name)), '-dpdf');
end
% Plot: Calculated Tdm vs. process parameters
for im=0:1:0
    range = [1e-3 1e-2 1e-1 1e0];
    
    [P_subplot,P_fig,P_colorbar] = subplot_position(460,380,1,1,2,5,5,30,12,50,50);
    figure('Name',char(strcat(image_name,{' '},'calculated {\it T}_d_m')),'NumberTitle','off','Position',P_fig);
    subplot('position',P_subplot{1});
    scatter3(X(:), Y(:), Z(:), 12, Tdm_calc(:), 'filled','MarkerEdgeAlpha', 1, 'MarkerFaceAlpha', 1);
    view(-82.221699999999998,32.591788594164463); 
    grid on;
    set(gca,'XMinorTick','on','ZMinorTick','on');
    set(gca,'xscale','log');
    h = xlabel('{\it T}_d_e_l_a_y [s]'); 
    h.Position = [0.035984152162232,-0.077100576068925,-0.503587218538343];
    h = ylabel('{\it n} [/]');
    h.Position = [0.000738124883731,4.318736427958505,-0.573604032950502];
    ylim([min(n_v) max(n_v)]);
    h = zlabel('{\it b} [/]');
    h.Position = [0.000922042789652,8.602126806647092,3.57627869e-7];
    zticks([-0.5 -0.25 0 0.25 0.5]);
    
    colormap(hsv);
    P_colorbar(1) = P_colorbar(1)-0.01;
    P_colorbar(2) = P_colorbar(2)+0.125;
    c = colorbar('vert','Position',P_colorbar);
    set(gca,'ColorScale','log');
    caxis([min(range) max(range)]);
    set(c, 'ticks', range);
    title(c,'{\it T}_d_m [s]','FontWeight','Normal');
    
    h = title('Calculated {\it T}_d_m vs. process parameters', 'FontWeight','Normal');
    h.Position(1) = h.Position(1)-0.2;

    tightfigadv();
    
    img_save_name = strcat(char(image_name),'_','Tdm');
    export_figure_as_vecgra([], fullfile(pwd, image_dir, char(img_save_name)), '-dmeta');
    export_figure_as_vecgra([], fullfile(pwd, image_dir, char(img_save_name)), '-dpdf');
end
% Plot: Criterion IAE vs. process parameters
for im=0:1:0   
    [P_subplot,P_fig,P_colorbar] = subplot_position(460,380,1,1,2,5,5,30,12,50,50);
    figure('Name',char(strcat(image_name,{' '},'calculated IAE')),'NumberTitle','off','Position',P_fig);
    subplot('position',P_subplot{1});
    scatter3(X(:), Y(:), Z(:), 12, IAE_2DZ_Tdm(:), 'filled','MarkerEdgeAlpha', 1, 'MarkerFaceAlpha', 1);
    view(-82.221699999999998,32.591788594164463); 
    grid on;
    set(gca,'XMinorTick','on','ZMinorTick','on');
    set(gca,'xscale','log');
    h = xlabel('{\it T}_d_e_l_a_y [s]'); 
    h.Position = [0.035984152162232,-0.077100576068925,-0.503587218538343];
    h = ylabel('{\it n} [/]');
    h.Position = [0.000738124883731,4.318736427958505,-0.573604032950502];
    ylim([min(n_v) max(n_v)]);
    h = zlabel('{\it b} [/]');
    h.Position = [0.000922042789652,8.602126806647092,3.57627869e-7];
    zticks([-0.5 -0.25 0 0.25 0.5]);
    
    colormap(jet);
    P_colorbar(1) = P_colorbar(1)-0.01;
    P_colorbar(2) = P_colorbar(2)+0.125;
    c = colorbar('vert','Position',P_colorbar);
    title(c,'IAE [/]','FontWeight','Normal');
    
    h = title('Criterion IAE vs. process parameters', 'FontWeight','Normal');
    h.Position(1) = h.Position(1)-0.15;

    tightfigadv();
    
     img_save_name = strcat(char(image_name),'_','IAE');
     export_figure_as_vecgra([], fullfile(pwd, image_dir, char(img_save_name)), '-dmeta');
     export_figure_as_vecgra([], fullfile(pwd, image_dir, char(img_save_name)), '-dpdf');
end
% Plot: Criterion IAE absolute difference vs. process parameters
for im=0:1:0
    IAE_diff = IAE_2D-IAE_2DZ_Tdm;
    range = [1e-4 1e-3 1e-2 1e-1 1e0];
    
    [P_subplot,P_fig,P_colorbar] = subplot_position(460,380,1,1,2,5,5,30,12,50,50);
    figure('Name',char(strcat(image_name,{' '},'calculated IAE')),'NumberTitle','off','Position',P_fig);
    subplot('position',P_subplot{1});
    scatter3(X(:), Y(:), Z(:), 12, IAE_diff(:), 'filled','MarkerEdgeAlpha', 1, 'MarkerFaceAlpha', 1);
    view(-82.221699999999998,32.591788594164463); 
    grid on;
    set(gca,'XMinorTick','on','ZMinorTick','on');
    set(gca,'xscale','log');
    h = xlabel('{\it T}_d_e_l_a_y [s]'); 
    h.Position = [0.035984152162232,-0.077100576068925,-0.503587218538343];
    h = ylabel('{\it n} [/]');
    h.Position = [0.000738124883731,4.318736427958505,-0.573604032950502];
    ylim([min(n_v) max(n_v)]);
    h = zlabel('{\it b} [/]');
    h.Position = [0.000922042789652,8.602126806647092,3.57627869e-7];
    zticks([-0.5 -0.25 0 0.25 0.5]);
    
    colormap(jet);
    P_colorbar(1) = P_colorbar(1)-0.01;
    P_colorbar(2) = P_colorbar(2)+0.125;
    c = colorbar('vert','Position',P_colorbar);
    set(gca,'ColorScale','log','YMinorTick', 'on');
    lim = caxis; lim(1) = min(range);
    caxis(lim);
    set(c, 'ticks', range);
    title(c,'IAE [/]','FontWeight','Normal');
    
    h = title('Criterion IAE absolute difference vs. process parameters', 'FontWeight','Normal');
    h.Position(1) = h.Position(1)-0.15;
    
    tightfigadv();
    
    img_save_name = strcat(char(image_name),'_','IAEdiff');
    export_figure_as_vecgra([], fullfile(pwd, image_dir, char(img_save_name)), '-dmeta');
    export_figure_as_vecgra([], fullfile(pwd, image_dir, char(img_save_name)), '-dpdf');
end
