% This .m file tests proposed model identification method in the
% time-domain under process noise.
% Used in Section 4.2 of the paper.
%% Init
clear all;
close all;

% Additionals folders
addpath('func_MAIN');
addpath('func_comparison');
addpath('SubScripts');
addpath('func_plot');
addpath('func_ExportFigure');
addpath('func_MOMI');

% Image save
image_dir = '_Image_export_emf_';
image_name = 'Test_Noise';

% Simulation parameters
iter = 200;
Ts = 0.001;
t_step = 2;
t_start = 0;
t_integ = 15;
t_avg = 17;
t_stop = 20;
Noise_amp_arr = [2e-2];

% Filter parameters
Tf_n = 0.1;
F_tf = tf([1],[Tf_n^2/4 Tf_n 1]);

% Plot parameters
display = {'Gp7'};
plot_compare = 'no';
plot_legend_str = 'Noise amplitude level';

% Process
num=[-0.5 1];den=[1 2 1];Tdelay=1;
tdomain_simulate_process_noise;

%% Open-loop responses (with noise)  --------------------------------------
rng(140);
noise = randn(size(u));

y_noise_arr = [];
for n=1:1:length(Noise_amp_arr)
    %add noise to signal
    y_noise_arr = [y_noise_arr y_process+(noise*Noise_amp_arr(n))];
end
y_noise_arr = [y_noise_arr y_process];

tdomain_plot_time_response_noise;

%% Histograms of noise ----------------------------------------------------
img_name = strcat(char(image_name),'_','histogram_noise_levels','_');

A_N_arr = NaN(length(Noise_amp_arr)*iter,6);
M_N_arr = NaN(length(Noise_amp_arr)*iter,6);

for i=1:1:iter
    rng(200+i);
    noise = randn(size(t));
    
    for n=1:1:length(Noise_amp_arr)
        % Add noise to signals
        y_noise = y_process+(noise*Noise_amp_arr(n));
        
        % Filter
        [y_f] = lsim(F_tf,y_noise,t); % signal u already filtered
        
        % Calculate areas
        [A0_n,A1_n,A2_n,A3_n,A4_n,A5_n] = meas2areas_2(y_f,u_f,t,t_start,t_integ, t_avg,t_stop);
        A_N_arr((n-1)*iter+i,:) = [A0_n,A1_n,A2_n,A3_n,A4_n,A5_n];
        
        % Estimate model parameters
        [Kpr_n,a1_n,a2_n,b1_n,Tdm_n,error_n] = area2Model_2DZ_Tdm(A0_n,A1_n,A2_n,A3_n,A4_n,process_type);
        M_N_arr((n-1)*iter+i,:) = [Kpr_n,a1_n,a2_n,b1_n,Tdm_n,0];
    end
end
clearvars A0_n A1_n A2_n A3_n A4_n A5_n Kpr_n a1_n a2_n b1_n Tdm_n error_n i

% Calculate standard deviation and relative mean value
for im=0:1:0
    % Areas
    std_A_arr = [];
    mean_A_arr = [];
    for n=1:1:length(Noise_amp_arr)
        std_A_arr = [std_A_arr; std(A_N_arr(iter*(n-1)+1:iter*n,1:5))];
        mean_A_arr = [mean_A_arr; mean(A_N_arr(iter*(n-1)+1:iter*n,1:5))];
    end
    rel_dev_A_arr = (mean_A_arr-[A0 A1 A2 A3 A4])./[A0 A1 A2 A3 A4];
    
    s = char(strcat('Standard deviation of Areas [A0 A1 A2 A3 A4]:'));
    disp(s);
    s = char(num2str(std_A_arr));
    disp(s);
    
    s = char(strcat('Relative mean value deviation of Areas [A0 A1 A2 A3 A4]:'));
    disp(s);
    s = char(num2str(rel_dev_A_arr));
    disp(s);
    clearvars std_A_arr mean_A_arr rel_dev_A_arr s
    
    % Model parameters
    std_M_arr = [];
    mean_M_arr = [];
    for n=1:1:length(Noise_amp_arr)
        std_M_arr = [std_M_arr; std(M_N_arr(iter*(n-1)+1:iter*n,1:5))];
        mean_M_arr = [mean_M_arr; mean(M_N_arr(iter*(n-1)+1:iter*n,1:5))];
    end
    rel_dev_M_arr = (mean_M_arr-[Kpr_t a1_t a2_t b1_t Tdm_t])./[Kpr_t a1_t a2_t b1_t Tdm_t];
    
    s = char(strcat('Standard deviation of Model parameters [Kpr a1 a2 b1 Tdm]:'));
    disp(s);
    s = char(num2str(std_M_arr));
    disp(s);
    
    s = char(strcat('Relative mean value deviation of Model parameters [Kpr a1 a2 b1 Tdm]:'));
    disp(s);
    s = char(num2str(rel_dev_M_arr));
    disp(s);
    clearvars std_M_arr mean_M_arr rel_dev_M_arr s
end

var_names_A = {'{\it A}_0','{\it A}_1','{\it A}_2','{\it A}_3','{\it A}_4'};
var_names_M = {'{\it K}_P_R','{\it a}_1','{\it a}_2','{\it b}_1','{\it T}_d_m'};
if  1 == length(Noise_amp_arr)
    histogram_1_plot(A_N_arr,iter,var_names_A,plot_legend_arr,image_dir,strcat(char(img_name),'Areas'));
    histogram_1_plot(M_N_arr,iter,var_names_M,plot_legend_arr,image_dir,strcat(char(img_name),'Model parameters'));
end
clearvars var_names_A var_names_M

%% Nyquist and Bode plots of models estimated from noisy signals -------------------
Ptf_data = [];
for n=1:1:length(Noise_amp_arr)
    method = plot_legend_arr(n);
    tmp_array = [M_N_arr(((n-1)*iter)+1:n*iter,:);[Kpr_t a1_t a2_t b1_t Tdm_t 0]];
    
    for i=1:1:length(tmp_array)
        tmp = tmp_array(i,:);
        tmp_tf = tf([tmp(1)*tmp(4) tmp(1)],[tmp(3) tmp(2) 1],'InputDelay',tmp(5));
        Ptf_data = [Ptf_data; tmp_tf];
    end
    
    tdomain_plot_nyquist_noise;
    tdomain_plot_bode_noise;
end
clearvars n method tmp_array tmp tmp_tf Ptf_data

%% Closed-loop responses --------------------------------------------------
% Simulation parameters
param_text = '[Kp Ki Kd]';
Ts = 0.001;
r = 1;
Dist = 1;
Tsim_ref_stop = 15;
Tsim_dist_stop = 15;

% MOMI parameters
KpPIMax = 100;
Tf = 0.1;
[A0_Tf,A1_Tf,A2_Tf,A3_Tf,A4_Tf,A5_Tf] = filtermodel2area(Tf);

for n=1:1:length(Noise_amp_arr)
    prepare_CLsimulation_variables;
    
    method = plot_legend_arr(n);
    CL_array = M_N_arr(((n-1)*iter)+1:n*iter,:);
    CL_array = [CL_array; [Kpr_t a1_t a2_t b1_t Tdm_t 0]];
    
    legend_s = {char(plot_legend_arr(n)),'Without noise'};
    method = num2str(n);
    
    % CL simulation
    for i=1:1:length(CL_array)
        tmp = CL_array(i,:);
        [A0_tmp,A1_tmp,A2_tmp,A3_tmp,A4_tmp,A5_tmp] = numden2areas ([tmp(1)*tmp(4) tmp(1)],[tmp(3) tmp(2) 1],tmp(5));
        
        [A0s,A1s,A2s,A3s,A4s,A5s] = multiarea(A0_tmp,A1_tmp,A2_tmp,A3_tmp,A4_tmp,A5_tmp,A0_Tf,A1_Tf,A2_Tf,A3_Tf,A4_Tf,A5_Tf);
        [Ki,Kp,Kd] = Area2PID (A0s,A1s,A2s,A3s,A4s,A5s,KpPIMax);
        param = [Kp Ki Kd];
        simulateCL_print;
    end
    
    plot_simulation_noise_response_2;
end
clearvars param_text Ts r Dist Tsim_ref_stop Tsim_dist_stop KpPIMax Tf A0_Tf A1_Tf A2_Tf A3_Tf A4_Tf A5_Tf
clearvars method CL_array legend_s method tmp A0_tmp A1_tmp A2_tmp A3_tmp A4_tmp A5_tm A0s A1s A2s A3s A4s A5s
clearvars Kp Ki Kd param