function export_figure_as_vecgra(fh, filename, format0, axh)
% function export_figure_as_vecgra(fh, filename, format0, axh)
% 
% EXPORTS A FIGURE TO A VECTOR GRAPHICS (PDF, EPS, EMF, SVG, PS) FILE.
% 
% INPUTS:
% fh:       Handle to figure. If omitted (or empty[]) the current figure is exported.
% filename: A name for the file, without the extension. If omitted (or empty[]) a dialog asks for the name and format.
% format0:  A vector graphics format to export the figure (see possible values below). If omitted (or empty[]) a dialog
%             allows to choose the format.
% axh:      Handle to axis in figure. If omitted, the whole figure is printed. 
%             If there is a value, it must be a handle to the axis to be exported.
%             If empty[], then the current axis is exported.
% 
% Advise: possibly the best performance is to export the figure or axis to SVG, and if needed edit and export it to a different format, 
%   using suitable software, like Inkscape (https://inkscape.org/), which is free and open-source.
% 
% Based on:
%  FIGURE2PDF (https://uk.mathworks.com/matlabcentral/fileexchange/71053-figure2pdf) Export the current MATLAB figure to a PDF file.
% 
% --------------------------------------------------------------------------------
%  OPTIONS FOR format0                                              FILE EXTENSION
% '-dpdf'   Full page Portable Document Format (PDF) color          .pdf
% '-deps'   Encapsulated PostScript (EPS) Level 3 black and white   .eps
% '-depsc'	Encapsulated PostScript (EPS) Level 3 color             .eps
% '-deps2'	Encapsulated PostScript (EPS) Level 2 black and white   .eps
% '-depsc2'	Encapsulated PostScript (EPS) Level 2 color             .eps
% '-dmeta'	Enhanced Metafile (Windows® only)                       .emf
% '-dsvg'   Scalable Vector Graphics (SVG)                          .svg
% '-dps'    Full-page PostScript (PS) Level 3 black and white       .ps
% '-dpsc'   Full-page PostScript (PS) Level 3 color                 .ps
% '-dps2'   Full-page PostScript (PS) Level 2 black and white       .ps
% '-dpsc2'	Full-page PostScript (PS) Level 2 color                 .ps
% 
% --------------------------------------------------------------------------------
% EXAMPLES:
% export_figure_as_vecgra                             : exports the current figure (complete), asking first for a file name and format.
% export_figure_as_vecgra(f1)                         : exports the figure with handle 'f1' (complete), asking first for a file name and format.
% export_figure_as_vecgra([], 'myfile')               : exports the current figure (complete), with the given file name and asking for a format.
% export_figure_as_vecgra([], 'myfile', '-dpdf')      : exports the current figure (complete) as PDF, with the given file name and format.
% export_figure_as_vecgra([], [], [], [])             : exports the current axis, asking first for a file name and format.
% export_figure_as_vecgra([], 'myfile', '-dsvg', [])  : exports the current axis as SVG, with the given file name and format.
% export_figure_as_vecgra(f1, 'myfile', '-dmeta', ax3) : exports the axis with handle 'ax3' of figure with handle 'f1' as EMF, 
%                                                         asking first for a file name and format.


if nargin < 4                               % No handle to axis is given. In that case it exports the whole figure.
  % axh = [];
  axh0 = gca;
else
  if isempty(axh)
    axh = gca;
  end
  axh0 = axh;
end
if nargin < 3                               % No format is given. In that case, PDF format is chosen.
  format0 = [];
end
if nargin < 2
  filename = [];
end
if nargin < 1
  fh = gcf;
elseif isempty(fh)
  fh = gcf;
end

set(axh0,  'LooseInset', get(axh0, 'TightInset'));
set(fh,  'Units', 'inches');
pos = get(fh,  'Position');
set(fh,  'PaperPositionMode',  'auto', 'PaperUnits', 'inches', ...
  'PaperPosition',  [0, 0, pos(3), pos(4)], 'PaperSize',  [pos(3), pos(4)]);

formats = {'-dpdf'; '-deps'; '-depsc'; '-deps2'; '-depsc2'; '-dmeta'; '-dsvg'; '-dps'; '-dpsc'; '-dps2'; '-dpsc2'};
filter = {'*.pdf',  'Portable document format (Adobe Acrobat)'; ...
          '*.eps',  'Enhanced PostScript level 3 B/W'; ...
          '*.eps',  'Enhanced PostScript level 3 C'; ...
          '*.eps',  'Enhanced PostScript level 2 B/W'; ...
          '*.eps',  'Enhanced PostScript level 2 C'; ...
          '*.emf',  'Enhanced metafile (Windows)'; ...
          '*.svg',  'Scalable Vector Graphics'; ...
          '*.ps',   'PostScript level 3 B/W'; ...
          '*.ps',   'PostScript level 3 C'; ...
          '*.ps',   'PostScript level 2 B/W'; ...
          '*.ps',   'PostScript level 2 C'};

if isempty(format0)
  if isempty(filename)                      % No file name nor format are given: a dialogue window asks for name and format.
    [filenameSAVE, pathname, index] = uiputfile(filter, 'Enter the name of the file to save the figure in');
    if filenameSAVE==0, error('NO NAME GIVEN TO THE FILE'); end
  else                                      % File name is given, but not the format.
    [filenameSAVE, pathname, index] = uiputfile(filter, 'Choose the format to save the figure in', filename);
    if filenameSAVE==0, error('NO FORMAT GIVEN TO THE FILE'); end
  end
  fullfilename = sprintf('%s%s', pathname, filenameSAVE);
  format0 = formats{index};
else
  % index = find(cellfun(@(x) any(strcmp(x, format0)), formats));   % https://stackoverflow.com/questions/28658346/finding-string-in-cell-array-of-cell-arrays
  if isempty(filename)                      % Format is given, but not the file name: a dialogue window asks for name.
    [filenameSAVE, pathname] = uiputfile('*.*', 'Enter the name of the file to save the figure in');
    if filenameSAVE==0, error('NO NAME GIVEN TO THE FILE'); end
    fullfilename = sprintf('%s%s', pathname, filenameSAVE);
  else                                      % Both the file name and the format are given.
    fullfilename = filename;
  end
end

if nargin < 4                               % No handle to axis is given. In that case the whole figure is exported.
  print(fh, fullfilename, format0, '-r0', '-painters'); %, 'renderer', 'painters');
else
  fh_new = figure('Visible', 'off');        % Creates a new figure, where the axis are copied.
  axh_new = copyobj(axh, fh_new);
  drawnow
  
  % pos_new = get(fh_new,  'Position');
  % set(fh_new,  'PaperPositionMode',  'auto', 'PaperUnits', 'inches', 'PaperPosition',  [0, 0, pos_new(3), pos_new(4)], 'PaperSize',  [pos_new(3), pos_new(4)]);
  set(axh_new, 'Units', 'normalized')
  set(axh_new, 'OuterPosition', [0 0 0.95 0.95])
  set(axh_new, 'PlotBoxAspectRatioMode', 'manual')
  set(axh_new, 'PlotBoxAspectRatio', [1 1 1])

  set(axh_new,  'LooseInset', get(axh_new, 'TightInset'));
  set(fh_new,  'Units', 'inches');
  pos_new = get(fh_new,  'Position');
  set(fh_new,  'PaperPositionMode',  'auto', 'PaperUnits', 'inches', ...
    'PaperPosition',  [0, 0, pos_new(3), pos_new(4)], 'PaperSize',  [pos_new(3), pos_new(4)]);

  % set(axh_new, 'Position', get(0, 'DefaultAxesPosition'));
  % saveas(fh_new, fullfilename, format0(3:end));
  print(fh_new, fullfilename, format0, '-r0', '-painters'); % , 'renderer', 'painters')
  close(fh_new)
end





