function [Kpr,a1,a2,b1,Tdm,error_b] = area2Model_2DZ_Tdm(A0,A1,A2,A3,A4,process_type)
%function [Kpr,a1,a2,b1,Tdm,error_b] = area2Model_2DZ_Tdm(A0,A1,A2,A3,A4,process_type);
%
% Function calculates second-order process model parameters from
% characteristic areas.
% Kpr*(1 + b*s)*e-Tdm*s
% ---------------------
%   a2*s^2 + a1*s + 1
%
% INPUTS:
% A0-A4:        Characteristic areas.
% process_type: Type of the process: "minimum phase", "non-minimum phase"
%
% OUTPUTS:
% Kpr:      Process model steady-state gain
% a1:       Process model time constant
% a2:       Process model time constant
% b1:       Process model zero
% Tdm:      Process model time-delay
% error_b:	Errors binary (TRUE, FALSE) vector
%   error_b(1): Tdm0 set to 0.
%   error_b(2): Tdm cannot be calculated from sixt-order polynomial
%   error_b(3): Process model not calculated.
%   error_b(4): Multiple process model solution found.
%   error_b(5): Calculation loop run twice.
%   error_b(6): Process type used for calculation.

error_b = false(1, 6);

if nargin < 6 %process type not specified
  process_type = "minimum phase";
else
  if process_type ~= "minimum phase" && process_type ~= "non-minimum phase"
    msg = 'Process type not defined correctly!';
    error(msg)
  end
end

%% Calculate Kpr ----------------------------------------------------------
Kpr = A0;

%% Get Tdm critical -------------------------------------------------------
Lv = roots([1/6,-A1/(2*A0),A1^2/A0^2-A2/A0,2*A1*A2/A0^2-A3/A0-A1^3/A0^3]);

v1 = abs(imag(Lv))<0.01*abs(real(Lv));
v2 = abs(imag(Lv))<1e-3*abs(A1/A0);
Lv = Lv( v1 | v2 );

Lv = real(Lv);
Lv = Lv(Lv >= 0);
Lv( Lv<5e-4*abs(A1/A0) ) = 0; % Tdm too small
if ( 0==length(Lv) )
    Lv = 0;
    error_b(1) = true;
end
Tdm_c = min(Lv);  

%% Calculate Tdm ----------------------------------------------------------
z6 = A0^3;
z5 = (-6*A0^2*A1);
z4 = 6*(-A2*A0^2 + 3*A0*A1^2);
z3 = 24*(A3*A0^2 - A1^3);
z2 = 72*(A4*A0^2 - 2*A3*A0*A1 + A2*A1^2);
z1 = 144*(A3*A1^2 - A1*A2^2 - A0*A4*A1 + A0*A3*A2);
z0 = 144*(A4*A1^2 - 2*A1*A2*A3 + A2^3 - A0*A4*A2 + A0*A3^2);

% Get roots
Lv = roots([z6,z5,z4,z3,z2,z1,z0]);
clearvars z6 z5 z4 z3 z2 z1 z0

% Get real solutions
v1 = abs(imag(Lv))<0.01*abs(real(Lv));
v2 = abs(imag(Lv))<1e-3*abs(A1/A0);
Lv = Lv( v1 | v2 );

Lv = real(Lv);
Lv = Lv(Lv >= 0);
Lv( Lv<5e-4*abs(A1/A0) ) = 0; % Tdm too small

[C, ia] = unique(round(Lv,4));
Lv = Lv( ia );

if ( 0==length(Lv) )
	error_b(2) = true;
end

%% calculate other parameters
for i=0:1:1

    if ( error_b(2) | error_b(3) )
        Tdm = Tdm_c;
        if ( error_b(3) )
            error_b(5) = true;
        end
    else
        Tdm = Lv;
    end

% calculate b1 -----------------------------------------------------------
    b1 = -(- A0^3*Tdm.^3 + 3*A0^2*A1*Tdm.^2 + 6*A2*A0^2*Tdm + 6*A3*A0^2 - 6*A0*A1^2*Tdm ...
        - 12*A2*A0*A1 + 6*A1^3)./(3*A0^3*Tdm.^2 - 6*A0^2*A1*Tdm - 6*A2*A0^2 + 6*A0*A1^2);
    b1(abs(b1)<1e-6) = 0;

% calculate a1 -----------------------------------------------------------
    a1 = (A1 + A0*b1 - A0*Tdm)/A0;

% calculate a2 ------------------------------------------------------------
    a2 = -(A2 + A0*(- Tdm.^2/2 + b1.*Tdm) - (A1*(A1 + A0*b1 - A0*Tdm))/A0)/A0;
    if ( all((a2<0)) ) % Check if all a2 negative or first order model
        a2 = zeros(length(a2),1);
    end
    a2(abs(a2./a1.^2)<1e-6) = 0;

% get appropriate results ------------------------------------------------
    % only solutions that ensures stabil process
    cond = (a1>=0 & a2>=0);
    Tdm = Tdm(cond);
    b1 = b1(cond);
    a1 = a1(cond);
    a2 = a2(cond);
  
    % solutions with right Tdm
    if  ( 1<length(Tdm) )      
        if ( "minimum phase" == process_type )
            cond = ( round(Tdm,4)>=round(Tdm_c,4) );
        else
            cond = ( round(Tdm,4)<=round(Tdm_c,4) );
        end
        Tdm = Tdm(cond);
        b1 = b1(cond);
        a1 = a1(cond);
        a2 = a2(cond);
        error_b(6) = true;
    else
        error_b(6) = false;
    end

    if ( 0==length(Tdm) ) % no solution
        error_b(3) = true;
        Tdm = NaN;
        b1 = NaN;
        a1 = NaN;
        a2 = NaN;
    else % solution found
        error_b(3) = false;
        break;
    end

end

if  ( 1<length(Tdm) )
    error_b(4) = true;
end

end