% function [A0,A1,A2,A3,A4,A5] = meas2areas (y,u,t,Tstart,Tstop);
%
% function meas2areas.m calculates the characteristic areas
% from the process response and its first, second ... fifth integral.
%
% y - process response
% u - process input
% t - time vector
% T0 - a start of experiment before the change of control variable (u) 
%      (for the calculation of starting mean value)
% T1 - time instant when control variable changes
% Tint - integration time
% Tfin - a time till when the final mean value is calculated
%
% A0..A5 - areas calculated from the process response

function [A0,A1,A2,A3,A4,A5] = meas2areas (y,u,t,Tstart,Tstop);

% Initialisation
a = max(size(y));
k = 1;

% Search for index of Tstart
while (t(k) < Tstart)
  k = k+1;
end

if (k > 1)
   kstart = k-1;
else
   kstart = k;
end

% Search for the change of the input variable (time instant and index)
u1 = u(kstart);
while (u(k) == u1) & (k < a)
   k = k+1;
end
kchange = k;
Tchange = t(kchange);

% Calculation of the lenth of experiment and integration time
Experiment_length = Tstop-Tchange;
Tinteg = Tstop - 0.1*Experiment_length;

while (t(k) < Tinteg) & (k < a)
  k = k+1;
end
kinteg = k;
Tinteg = t(kinteg);

while (t(k) < Tstop) & (k < a)
  k = k+1;
end
kstop = k;

% Calculation of the mean values of the signals before change and after the process settles
ya0=mean(y(kstart:kchange));
ya1=mean(y(kinteg:kstop));
ua0=mean(u(kstart:kchange-1));
ua1=mean(u(kinteg:kstop));

DU = ua1-ua0;				% The amplitude of the process input change (u(inf)-u(0))

A0 = (ya1-ya0)/DU;		% Process gain
y = (y-ya0)/DU;			% Normalised response		
u = (u-ua0)/DU;

dtv = t(kchange+1:kinteg)-t(kchange:kinteg-1);		% Sampling times

% Calculation of the multiple integrals of the process input signal
Iu1 = [0;dtv.*cumsum((u(kchange:kinteg-1)+u(kchange+1:kinteg))/2)];
%Iu1 = [0;dtv(1:kinteg-kchange).*cumsum(u(kchange:kinteg))];
Iu2 = [0;dtv.*cumsum((Iu1(1:kinteg-kchange)+Iu1(2:kinteg-kchange+1))/2)];
Iu3 = [0;dtv.*cumsum((Iu2(1:kinteg-kchange)+Iu2(2:kinteg-kchange+1))/2)];
Iu4 = [0;dtv.*cumsum((Iu3(1:kinteg-kchange)+Iu3(2:kinteg-kchange+1))/2)];
Iu5 = [0;dtv.*cumsum((Iu4(1:kinteg-kchange)+Iu4(2:kinteg-kchange+1))/2)];

% Calculation of the multiple integrals of the process output signal
Iy1 = [0;dtv.*cumsum((y(kchange:kinteg-1)+y(kchange+1:kinteg))/2)];
Iy2 = [0;dtv.*cumsum((Iy1(1:kinteg-kchange)+Iy1(2:kinteg-kchange+1))/2)];
Iy3 = [0;dtv.*cumsum((Iy2(1:kinteg-kchange)+Iy2(2:kinteg-kchange+1))/2)];
Iy4 = [0;dtv.*cumsum((Iy3(1:kinteg-kchange)+Iy3(2:kinteg-kchange+1))/2)];
Iy5 = [0;dtv.*cumsum((Iy4(1:kinteg-kchange)+Iy4(2:kinteg-kchange+1))/2)];

% Calculation of areas
y1 = (A0*Iu1-Iy1);
A1 = y1(kinteg-kchange);
y2 = (A1*Iu1-A0*Iu2+Iy2);
A2 = y2(kinteg-kchange);
y3 = (A2*Iu1-A1*Iu2+A0*Iu3-Iy3);
A3 = y3(kinteg-kchange);
y4 = (A3*Iu1-A2*Iu2+A1*Iu3-A0*Iu4+Iy4);
A4 = y4(kinteg-kchange);
y5 = (A4*Iu1-A3*Iu2+A2*Iu3-A1*Iu4+A0*Iu5-Iy5);
A5 = y5(kinteg-kchange);
end
